@@ -1,8 +1,5 @@
-use crate::utils::{replace_space, QS_ENCODE_SET};
-
 use super::*;
 
-use percent_encoding::percent_encode;
 use serde::de;
 
 use std::borrow::Cow;
@@ -28,17 +25,8 @@ impl<'a> Level<'a> {
         if let Level::Nested(ref mut map) = *self {
             match map.entry(key) {
                 Entry::Occupied(mut o) => {
-                    let key = o.key();
-                    let error = if key.contains('[') {
-                        let newkey = percent_encode(key.as_bytes(), QS_ENCODE_SET)
-                            .map(replace_space)
-                            .collect::<String>();
-                        format!("Multiple values for one key: \"{}\"\nInvalid field contains an encoded bracket -- did you mean to use non-strict mode?\n  https://docs.rs/serde_qs/latest/serde_qs/#strict-vs-non-strict-modes", newkey)
-                    } else {
-                        format!("Multiple values for one key: \"{}\"", key)
-                    };
                     // Throw away old result; map is now invalid anyway.
-                    let _ = o.insert(Level::Invalid(error));
+                    let _ = o.insert(Level::Invalid("Multiple values for one key"));
                 }
                 Entry::Vacant(vm) => {
                     // Map is empty, result is None
@@ -52,8 +40,7 @@ impl<'a> Level<'a> {
         } else {
             *self = Level::Invalid(
                 "Attempted to insert map value into \
-                 non-map structure"
-                    .to_string(),
+                 non-map structure",
             );
         }
     }
@@ -64,7 +51,7 @@ impl<'a> Level<'a> {
             match map.entry(key) {
                 Entry::Occupied(mut o) => {
                     // Throw away old result; map is now invalid anyway.
-                    let _ = o.insert(Level::Invalid("Multiple values for one key".to_string()));
+                    let _ = o.insert(Level::Invalid("Multiple values for one key"));
                 }
                 Entry::Vacant(vm) => {
                     // Map is empty, result is None
@@ -79,8 +66,7 @@ impl<'a> Level<'a> {
         } else {
             *self = Level::Invalid(
                 "Attempted to insert seq value into \
-                 non-seq structure"
-                    .to_string(),
+                 non-seq structure",
             );
         }
     }
@@ -99,8 +85,7 @@ impl<'a> Level<'a> {
         } else {
             *self = Level::Invalid(
                 "Attempted to insert seq value into \
-                 non-seq structure"
-                    .to_string(),
+                 non-seq structure",
             );
         }
     }
@@ -242,31 +227,24 @@ impl<'a> Parser<'a> {
     /// present.
     fn collect_str(&mut self) -> Result<Cow<'a, str>> {
         let replaced = replace_plus(&self.inner[self.acc.0..self.acc.1 - 1]);
-        let decoder = percent_encoding::percent_decode(&replaced);
-
-        let maybe_decoded = if self.strict {
-            decoder.decode_utf8()?
-        } else {
-            decoder.decode_utf8_lossy()
-        };
-
-        let ret: Result<Cow<'a, str>> = match maybe_decoded {
-            Cow::Borrowed(_) => {
-                match replaced {
-                    Cow::Borrowed(_) => {
-                        // In this case, neither method made replacements, so we
-                        // reuse the original bytes
-                        let res = str::from_utf8(&self.inner[self.acc.0..self.acc.1 - 1])?;
-                        Ok(Cow::Borrowed(res))
-                    }
-                    Cow::Owned(owned) => {
-                        let res = String::from_utf8(owned)?;
-                        Ok(Cow::Owned(res))
+        let ret: Result<Cow<'a, str>> =
+            match percent_encoding::percent_decode(&replaced).decode_utf8()? {
+                Cow::Borrowed(_) => {
+                    match replaced {
+                        Cow::Borrowed(_) => {
+                            // In this case, neither method made replacements, so we
+                            // reuse the original bytes
+                            let res = str::from_utf8(&self.inner[self.acc.0..self.acc.1 - 1])?;
+                            Ok(Cow::Borrowed(res))
+                        }
+                        Cow::Owned(owned) => {
+                            let res = String::from_utf8(owned)?;
+                            Ok(Cow::Owned(res))
+                        }
                     }
                 }
-            }
-            Cow::Owned(owned) => Ok(Cow::Owned(owned)),
-        };
+                Cow::Owned(owned) => Ok(Cow::Owned(owned)),
+            };
         self.clear_acc();
         ret.map_err(Error::from)
     }
