@@ -1,13 +1,129 @@
 //! Style types for CSS Grid layout
-use super::{AlignContent, LengthPercentage, Style};
+use super::{
+    AlignContent, AlignItems, AlignSelf, CompactLength, CoreStyle, Dimension, JustifyContent, LengthPercentage,
+    LengthPercentageAuto, Style,
+};
 use crate::compute::grid::{GridCoordinate, GridLine, OriginZeroLine};
-use crate::geometry::{AbsoluteAxis, AbstractAxis};
-use crate::geometry::{Line, MinMax};
+use crate::geometry::{AbsoluteAxis, AbstractAxis, Line, MinMax, Size};
 use crate::style_helpers::*;
 use crate::util::sys::GridTrackVec;
+use core::borrow::Borrow;
 use core::cmp::{max, min};
 use core::convert::Infallible;
 
+/// The set of styles required for a CSS Grid container
+pub trait GridContainerStyle: CoreStyle {
+    /// The type returned by grid_template_rows and grid_template_columns
+    type TemplateTrackList<'a>: Borrow<[TrackSizingFunction]>
+    where
+        Self: 'a;
+    /// The type returned by grid_auto_rows and grid_auto_columns
+    type AutoTrackList<'a>: Borrow<[NonRepeatedTrackSizingFunction]>
+    where
+        Self: 'a;
+
+    // FIXME: re-add default implemenations for grid_{template,auto}_{rows,columns} once the
+    // associated_type_defaults feature (https://github.com/rust-lang/rust/issues/29661) is stabilised.
+
+    /// Defines the track sizing functions (heights) of the grid rows
+    fn grid_template_rows(&self) -> Self::TemplateTrackList<'_>;
+    /// Defines the track sizing functions (widths) of the grid columns
+    fn grid_template_columns(&self) -> Self::TemplateTrackList<'_>;
+    /// Defines the size of implicitly created rows
+    fn grid_auto_rows(&self) -> Self::AutoTrackList<'_>;
+    /// Defined the size of implicitly created columns
+    fn grid_auto_columns(&self) -> Self::AutoTrackList<'_>;
+
+    /// Controls how items get placed into the grid for auto-placed items
+    #[inline(always)]
+    fn grid_auto_flow(&self) -> GridAutoFlow {
+        Style::DEFAULT.grid_auto_flow
+    }
+
+    /// How large should the gaps between items in a grid or flex container be?
+    #[inline(always)]
+    fn gap(&self) -> Size<LengthPercentage> {
+        Style::DEFAULT.gap
+    }
+
+    // Alignment properties
+
+    /// How should content contained within this item be aligned in the cross/block axis
+    #[inline(always)]
+    fn align_content(&self) -> Option<AlignContent> {
+        Style::DEFAULT.align_content
+    }
+    /// How should contained within this item be aligned in the main/inline axis
+    #[inline(always)]
+    fn justify_content(&self) -> Option<JustifyContent> {
+        Style::DEFAULT.justify_content
+    }
+    /// How this node's children aligned in the cross/block axis?
+    #[inline(always)]
+    fn align_items(&self) -> Option<AlignItems> {
+        Style::DEFAULT.align_items
+    }
+    /// How this node's children should be aligned in the inline axis
+    #[inline(always)]
+    fn justify_items(&self) -> Option<AlignItems> {
+        Style::DEFAULT.justify_items
+    }
+
+    /// Get a grid item's row or column placement depending on the axis passed
+    #[inline(always)]
+    fn grid_template_tracks(&self, axis: AbsoluteAxis) -> Self::TemplateTrackList<'_> {
+        match axis {
+            AbsoluteAxis::Horizontal => self.grid_template_columns(),
+            AbsoluteAxis::Vertical => self.grid_template_rows(),
+        }
+    }
+
+    /// Get a grid container's align-content or justify-content alignment depending on the axis passed
+    #[inline(always)]
+    fn grid_align_content(&self, axis: AbstractAxis) -> AlignContent {
+        match axis {
+            AbstractAxis::Inline => self.justify_content().unwrap_or(AlignContent::Stretch),
+            AbstractAxis::Block => self.align_content().unwrap_or(AlignContent::Stretch),
+        }
+    }
+}
+
+/// The set of styles required for a CSS Grid item (child of a CSS Grid container)
+pub trait GridItemStyle: CoreStyle {
+    /// Defines which row in the grid the item should start and end at
+    #[inline(always)]
+    fn grid_row(&self) -> Line<GridPlacement> {
+        Style::DEFAULT.grid_row
+    }
+    /// Defines which column in the grid the item should start and end at
+    #[inline(always)]
+    fn grid_column(&self) -> Line<GridPlacement> {
+        Style::DEFAULT.grid_column
+    }
+
+    /// How this node should be aligned in the cross/block axis
+    /// Falls back to the parents [`AlignItems`] if not set
+    #[inline(always)]
+    fn align_self(&self) -> Option<AlignSelf> {
+        Style::DEFAULT.align_self
+    }
+    /// How this node should be aligned in the inline axis
+    /// Falls back to the parents [`super::JustifyItems`] if not set
+    #[inline(always)]
+    fn justify_self(&self) -> Option<AlignSelf> {
+        Style::DEFAULT.justify_self
+    }
+
+    /// Get a grid item's row or column placement depending on the axis passed
+    #[inline(always)]
+    fn grid_placement(&self, axis: AbsoluteAxis) -> Line<GridPlacement> {
+        match axis {
+            AbsoluteAxis::Horizontal => self.grid_column(),
+            AbsoluteAxis::Vertical => self.grid_row(),
+        }
+    }
+}
+
 /// Controls whether grid items are placed row-wise or column-wise. And whether the sparse or dense packing algorithm is used.
 ///
 /// The "dense" packing algorithm attempts to fill in holes earlier in the grid, if smaller items come up later. This may cause items to appear out-of-order, when doing so would fill in holes left by larger items.
@@ -128,13 +244,6 @@ impl GridPlacement {
 }
 
 impl<T: GridCoordinate> Line<GenericGridPlacement<T>> {
-    #[inline]
-    /// Whether the track position is definite in this axis (or the item will need auto placement)
-    /// The track position is definite if least one of the start and end positions is a track index
-    pub fn is_definite(&self) -> bool {
-        matches!((self.start, self.end), (GenericGridPlacement::Line(_), _) | (_, GenericGridPlacement::Line(_)))
-    }
-
     /// Resolves the span for an indefinite placement (a placement that does not consist of two `Track`s).
     /// Panics if called on a definite placement
     pub fn indefinite_span(&self) -> u16 {
@@ -154,6 +263,18 @@ impl<T: GridCoordinate> Line<GenericGridPlacement<T>> {
 }
 
 impl Line<GridPlacement> {
+    #[inline]
+    /// Whether the track position is definite in this axis (or the item will need auto placement)
+    /// The track position is definite if least one of the start and end positions is a NON-ZERO track index
+    /// (0 is an invalid line in GridLine coordinates, and falls back to "auto" which is indefinite)
+    pub fn is_definite(&self) -> bool {
+        match (self.start, self.end) {
+            (GenericGridPlacement::Line(line), _) if line.as_i16() != 0 => true,
+            (_, GenericGridPlacement::Line(line)) if line.as_i16() != 0 => true,
+            _ => false,
+        }
+    }
+
     /// Apply a mapping function if the [`GridPlacement`] is a `Track`. Otherwise return `self` unmodified.
     pub fn into_origin_zero(&self, explicit_track_count: u16) -> Line<OriginZeroGridPlacement> {
         Line {
@@ -164,6 +285,13 @@ impl Line<GridPlacement> {
 }
 
 impl Line<OriginZeroGridPlacement> {
+    #[inline]
+    /// Whether the track position is definite in this axis (or the item will need auto placement)
+    /// The track position is definite if least one of the start and end positions is a track index
+    pub fn is_definite(&self) -> bool {
+        matches!((self.start, self.end), (GenericGridPlacement::Line(_), _) | (_, GenericGridPlacement::Line(_)))
+    }
+
     /// If at least one of the of the start and end positions is a track index then the other end can be resolved
     /// into a track index purely based on the information contained with the placement specification
     pub fn resolve_definite_grid_lines(&self) -> Line<OriginZeroLine> {
@@ -238,61 +366,186 @@ impl Default for Line<GridPlacement> {
 /// on the size of it's contents, the amount of available space, and the sizing constraint the grid is being size under.
 /// See <https://developer.mozilla.org/en-US/docs/Web/CSS/grid-template-columns>
 #[derive(Copy, Clone, PartialEq, Debug)]
-#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
-pub enum MaxTrackSizingFunction {
-    /// Track maximum size should be a fixed length or percentage value
-    Fixed(LengthPercentage),
-    /// Track maximum size should be content sized under a min-content constraint
-    MinContent,
-    /// Track maximum size should be content sized under a max-content constraint
-    MaxContent,
-    /// Track maximum size should be sized according to the fit-content formula
-    FitContent(LengthPercentage),
-    /// Track maximum size should be automatically sized
-    Auto,
-    /// The dimension as a fraction of the total available grid space (`fr` units in CSS)
-    /// Specified value is the numerator of the fraction. Denominator is the sum of all fraction specified in that grid dimension
-    /// Spec: <https://www.w3.org/TR/css3-grid-layout/#fr-unit>
-    Fraction(f32),
+#[cfg_attr(feature = "serde", derive(Serialize))]
+pub struct MaxTrackSizingFunction(pub(crate) CompactLength);
+impl TaffyZero for MaxTrackSizingFunction {
+    const ZERO: Self = Self(CompactLength::ZERO);
 }
 impl TaffyAuto for MaxTrackSizingFunction {
-    const AUTO: Self = Self::Auto;
+    const AUTO: Self = Self(CompactLength::AUTO);
 }
 impl TaffyMinContent for MaxTrackSizingFunction {
-    const MIN_CONTENT: Self = Self::MinContent;
+    const MIN_CONTENT: Self = Self(CompactLength::MIN_CONTENT);
 }
 impl TaffyMaxContent for MaxTrackSizingFunction {
-    const MAX_CONTENT: Self = Self::MaxContent;
-}
-impl TaffyFitContent for MaxTrackSizingFunction {
-    fn fit_content(argument: LengthPercentage) -> Self {
-        Self::FitContent(argument)
-    }
-}
-impl TaffyZero for MaxTrackSizingFunction {
-    const ZERO: Self = Self::Fixed(LengthPercentage::ZERO);
+    const MAX_CONTENT: Self = Self(CompactLength::MAX_CONTENT);
 }
 impl FromLength for MaxTrackSizingFunction {
     fn from_length<Input: Into<f32> + Copy>(value: Input) -> Self {
-        Self::Fixed(LengthPercentage::from_length(value))
+        Self::length(value.into())
     }
 }
 impl FromPercent for MaxTrackSizingFunction {
-    fn from_percent<Input: Into<f32> + Copy>(percent: Input) -> Self {
-        Self::Fixed(LengthPercentage::from_percent(percent))
+    fn from_percent<Input: Into<f32> + Copy>(value: Input) -> Self {
+        Self::percent(value.into())
     }
 }
-impl FromFlex for MaxTrackSizingFunction {
-    fn from_flex<Input: Into<f32> + Copy>(flex: Input) -> Self {
-        Self::Fraction(flex.into())
+impl TaffyFitContent for MaxTrackSizingFunction {
+    fn fit_content(argument: LengthPercentage) -> Self {
+        Self(CompactLength::fit_content(argument))
+    }
+}
+impl FromFr for MaxTrackSizingFunction {
+    fn from_fr<Input: Into<f32> + Copy>(value: Input) -> Self {
+        Self::fr(value.into())
+    }
+}
+impl From<LengthPercentage> for MaxTrackSizingFunction {
+    fn from(input: LengthPercentage) -> Self {
+        Self(input.0)
+    }
+}
+impl From<LengthPercentageAuto> for MaxTrackSizingFunction {
+    fn from(input: LengthPercentageAuto) -> Self {
+        Self(input.0)
+    }
+}
+impl From<Dimension> for MaxTrackSizingFunction {
+    fn from(input: Dimension) -> Self {
+        Self(input.0)
+    }
+}
+impl From<MinTrackSizingFunction> for MaxTrackSizingFunction {
+    fn from(input: MinTrackSizingFunction) -> Self {
+        Self(input.0)
+    }
+}
+#[cfg(feature = "serde")]
+impl<'de> serde::Deserialize<'de> for MaxTrackSizingFunction {
+    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
+    where
+        D: serde::Deserializer<'de>,
+    {
+        let inner = CompactLength::deserialize(deserializer)?;
+        // Note: validation intentionally excludes the CALC_TAG as deserializing calc() values is not supported
+        if matches!(
+            inner.tag(),
+            CompactLength::LENGTH_TAG
+                | CompactLength::PERCENT_TAG
+                | CompactLength::AUTO_TAG
+                | CompactLength::MIN_CONTENT_TAG
+                | CompactLength::MAX_CONTENT_TAG
+                | CompactLength::FIT_CONTENT_PX_TAG
+                | CompactLength::FIT_CONTENT_PERCENT_TAG
+                | CompactLength::FR_TAG
+        ) {
+            Ok(Self(inner))
+        } else {
+            Err(serde::de::Error::custom("Invalid tag"))
+        }
     }
 }
 
 impl MaxTrackSizingFunction {
+    /// An absolute length in some abstract units. Users of Taffy may define what they correspond
+    /// to in their application (pixels, logical pixels, mm, etc) as they see fit.
+    #[inline(always)]
+    pub const fn length(val: f32) -> Self {
+        Self(CompactLength::length(val))
+    }
+
+    /// A percentage length relative to the size of the containing block.
+    ///
+    /// **NOTE: percentages are represented as a f32 value in the range [0.0, 1.0] NOT the range [0.0, 100.0]**
+    #[inline(always)]
+    pub const fn percent(val: f32) -> Self {
+        Self(CompactLength::percent(val))
+    }
+
+    /// The dimension should be automatically computed according to algorithm-specific rules
+    /// regarding the default size of boxes.
+    #[inline(always)]
+    pub const fn auto() -> Self {
+        Self(CompactLength::auto())
+    }
+
+    /// The size should be the "min-content" size.
+    /// This is the smallest size that can fit the item's contents with ALL soft line-wrapping opportunities taken
+    #[inline(always)]
+    pub const fn min_content() -> Self {
+        Self(CompactLength::min_content())
+    }
+
+    /// The size should be the "max-content" size.
+    /// This is the smallest size that can fit the item's contents with NO soft line-wrapping opportunities taken
+    #[inline(always)]
+    pub const fn max_content() -> Self {
+        Self(CompactLength::max_content())
+    }
+
+    /// The size should be computed according to the "fit content" formula:
+    ///    `max(min_content, min(max_content, limit))`
+    /// where:
+    ///    - `min_content` is the [min-content](Self::min_content) size
+    ///    - `max_content` is the [max-content](Self::max_content) size
+    ///    - `limit` is a LENGTH value passed to this function
+    ///
+    /// The effect of this is that the item takes the size of `limit` clamped
+    /// by the min-content and max-content sizes.
+    #[inline(always)]
+    pub const fn fit_content_px(limit: f32) -> Self {
+        Self(CompactLength::fit_content_px(limit))
+    }
+
+    /// The size should be computed according to the "fit content" formula:
+    ///    `max(min_content, min(max_content, limit))`
+    /// where:
+    ///    - `min_content` is the [min-content](Self::min_content) size
+    ///    - `max_content` is the [max-content](Self::max_content) size
+    ///    - `limit` is a PERCENTAGE value passed to this function
+    ///
+    /// The effect of this is that the item takes the size of `limit` clamped
+    /// by the min-content and max-content sizes.
+    #[inline(always)]
+    pub const fn fit_content_percent(limit: f32) -> Self {
+        Self(CompactLength::fit_content_percent(limit))
+    }
+
+    /// The dimension as a fraction of the total available grid space (`fr` units in CSS)
+    /// Specified value is the numerator of the fraction. Denominator is the sum of all fraction specified in that grid dimension
+    /// Spec: <https://www.w3.org/TR/css3-grid-layout/#fr-unit>
+    #[inline(always)]
+    pub const fn fr(val: f32) -> Self {
+        Self(CompactLength::fr(val))
+    }
+
+    /// A `calc()` value. The value passed here is treated as an opaque handle to
+    /// the actual calc representation and may be a pointer, index, etc.
+    ///
+    /// The low 3 bits are used as a tag value and will be returned as 0.
+    #[inline]
+    #[cfg(feature = "calc")]
+    pub fn calc(ptr: *const ()) -> Self {
+        Self(CompactLength::calc(ptr))
+    }
+
+    /// Create a LengthPercentageAuto from a raw `CompactLength`.
+    /// # Safety
+    /// CompactLength must represent a valid variant for LengthPercentageAuto
+    #[allow(unsafe_code)]
+    pub unsafe fn from_raw(val: CompactLength) -> Self {
+        Self(val)
+    }
+
+    /// Get the underlying `CompactLength` representation of the value
+    pub fn into_raw(self) -> CompactLength {
+        self.0
+    }
+
     /// Returns true if the max track sizing function is `MinContent`, `MaxContent`, `FitContent` or `Auto`, else false.
     #[inline(always)]
     pub fn is_intrinsic(&self) -> bool {
-        matches!(self, Self::MinContent | Self::MaxContent | Self::FitContent(_) | Self::Auto)
+        self.0.is_intrinsic()
     }
 
     /// Returns true if the max track sizing function is `MaxContent`, `FitContent` or `Auto` else false.
@@ -300,25 +553,72 @@ impl MaxTrackSizingFunction {
     /// See: <https://www.w3.org/TR/css-grid-1/#algo-terms>
     #[inline(always)]
     pub fn is_max_content_alike(&self) -> bool {
-        matches!(self, Self::MaxContent | Self::FitContent(_) | Self::Auto)
+        self.0.is_max_content_alike()
     }
 
-    /// Returns true if the max track sizing function is `Flex`, else false.
+    /// Returns true if the an Fr value, else false.
     #[inline(always)]
-    pub fn is_flexible(&self) -> bool {
-        matches!(self, Self::Fraction(_))
+    pub fn is_fr(&self) -> bool {
+        self.0.is_fr()
+    }
+
+    /// Returns true if the is `Auto`, else false.
+    #[inline(always)]
+    pub fn is_auto(&self) -> bool {
+        self.0.is_auto()
+    }
+
+    /// Returns true if value is MinContent
+    #[inline(always)]
+    pub fn is_min_content(&self) -> bool {
+        self.0.is_min_content()
+    }
+
+    /// Returns true if value is MaxContent
+    #[inline(always)]
+    pub fn is_max_content(&self) -> bool {
+        self.0.is_max_content()
+    }
+
+    /// Returns true if value is FitContent(...)
+    #[inline(always)]
+    pub fn is_fit_content(&self) -> bool {
+        self.0.is_fit_content()
+    }
+
+    /// Returns true if value is MaxContent or FitContent(...)
+    #[inline(always)]
+    pub fn is_max_or_fit_content(&self) -> bool {
+        self.0.is_max_or_fit_content()
+    }
+
+    /// Returns whether the value can be resolved using `Self::definite_value`
+    #[inline(always)]
+    pub fn has_definite_value(self, parent_size: Option<f32>) -> bool {
+        match self.0.tag() {
+            CompactLength::LENGTH_TAG => true,
+            CompactLength::PERCENT_TAG => parent_size.is_some(),
+            #[cfg(feature = "calc")]
+            _ if self.0.is_calc() => parent_size.is_some(),
+            _ => false,
+        }
     }
 
     /// Returns fixed point values directly. Attempts to resolve percentage values against
     /// the passed available_space and returns if this results in a concrete value (which it
     /// will if the available_space is `Some`). Otherwise returns None.
     #[inline(always)]
-    pub fn definite_value(self, parent_size: Option<f32>) -> Option<f32> {
-        use MaxTrackSizingFunction::*;
-        match self {
-            Fixed(LengthPercentage::Length(size)) => Some(size),
-            Fixed(LengthPercentage::Percent(fraction)) => parent_size.map(|size| fraction * size),
-            MinContent | MaxContent | FitContent(_) | Auto | Fraction(_) => None,
+    pub fn definite_value(
+        self,
+        parent_size: Option<f32>,
+        calc_resolver: impl Fn(*const (), f32) -> f32,
+    ) -> Option<f32> {
+        match self.0.tag() {
+            CompactLength::LENGTH_TAG => Some(self.0.value()),
+            CompactLength::PERCENT_TAG => parent_size.map(|size| self.0.value() * size),
+            #[cfg(feature = "calc")]
+            _ if self.0.is_calc() => parent_size.map(|size| calc_resolver(self.0.calc_value(), size)),
+            _ => None,
         }
     }
 
@@ -329,31 +629,33 @@ impl MaxTrackSizingFunction {
     ///     - A fit-content sizing function with percentage argument (with definite available space)
     /// All other kinds of track sizing function return None.
     #[inline(always)]
-    pub fn definite_limit(self, parent_size: Option<f32>) -> Option<f32> {
-        use MaxTrackSizingFunction::FitContent;
-        match self {
-            FitContent(LengthPercentage::Length(size)) => Some(size),
-            FitContent(LengthPercentage::Percent(fraction)) => parent_size.map(|size| fraction * size),
-            _ => self.definite_value(parent_size),
+    pub fn definite_limit(
+        self,
+        parent_size: Option<f32>,
+        calc_resolver: impl Fn(*const (), f32) -> f32,
+    ) -> Option<f32> {
+        match self.0.tag() {
+            CompactLength::FIT_CONTENT_PX_TAG => Some(self.0.value()),
+            CompactLength::FIT_CONTENT_PERCENT_TAG => parent_size.map(|size| self.0.value() * size),
+            _ => self.definite_value(parent_size, calc_resolver),
         }
     }
 
     /// Resolve percentage values against the passed parent_size, returning Some(value)
     /// Non-percentage values always return None.
     #[inline(always)]
-    pub fn resolved_percentage_size(self, parent_size: f32) -> Option<f32> {
-        use MaxTrackSizingFunction::*;
-        match self {
-            Fixed(LengthPercentage::Percent(fraction)) => Some(fraction * parent_size),
-            Fixed(LengthPercentage::Length(_)) | MinContent | MaxContent | FitContent(_) | Auto | Fraction(_) => None,
-        }
+    pub fn resolved_percentage_size(
+        self,
+        parent_size: f32,
+        calc_resolver: impl Fn(*const (), f32) -> f32,
+    ) -> Option<f32> {
+        self.0.resolved_percentage_size(parent_size, calc_resolver)
     }
 
     /// Whether the track sizing functions depends on the size of the parent node
     #[inline(always)]
     pub fn uses_percentage(self) -> bool {
-        use MaxTrackSizingFunction::*;
-        matches!(self, Fixed(LengthPercentage::Percent(_)) | FitContent(LengthPercentage::Percent(_)))
+        self.0.uses_percentage()
     }
 }
 
@@ -363,80 +665,211 @@ impl MaxTrackSizingFunction {
 /// on the size of it's contents, the amount of available space, and the sizing constraint the grid is being size under.
 /// See <https://developer.mozilla.org/en-US/docs/Web/CSS/grid-template-columns>
 #[derive(Copy, Clone, PartialEq, Debug)]
-#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
-pub enum MinTrackSizingFunction {
-    /// Track minimum size should be a fixed length or percentage value
-    Fixed(LengthPercentage),
-    /// Track minimum size should be content sized under a min-content constraint
-    MinContent,
-    /// Track minimum size should be content sized under a max-content constraint
-    MaxContent,
-    /// Track minimum size should be automatically sized
-    Auto,
+#[cfg_attr(feature = "serde", derive(Serialize))]
+pub struct MinTrackSizingFunction(pub(crate) CompactLength);
+impl TaffyZero for MinTrackSizingFunction {
+    const ZERO: Self = Self(CompactLength::ZERO);
 }
 impl TaffyAuto for MinTrackSizingFunction {
-    const AUTO: Self = Self::Auto;
+    const AUTO: Self = Self(CompactLength::AUTO);
 }
 impl TaffyMinContent for MinTrackSizingFunction {
-    const MIN_CONTENT: Self = Self::MinContent;
+    const MIN_CONTENT: Self = Self(CompactLength::MIN_CONTENT);
 }
 impl TaffyMaxContent for MinTrackSizingFunction {
-    const MAX_CONTENT: Self = Self::MaxContent;
-}
-impl TaffyZero for MinTrackSizingFunction {
-    const ZERO: Self = Self::Fixed(LengthPercentage::ZERO);
+    const MAX_CONTENT: Self = Self(CompactLength::MAX_CONTENT);
 }
 impl FromLength for MinTrackSizingFunction {
     fn from_length<Input: Into<f32> + Copy>(value: Input) -> Self {
-        Self::Fixed(LengthPercentage::from_length(value))
+        Self::length(value.into())
     }
 }
 impl FromPercent for MinTrackSizingFunction {
-    fn from_percent<Input: Into<f32> + Copy>(percent: Input) -> Self {
-        Self::Fixed(LengthPercentage::from_percent(percent))
+    fn from_percent<Input: Into<f32> + Copy>(value: Input) -> Self {
+        Self::percent(value.into())
+    }
+}
+impl From<LengthPercentage> for MinTrackSizingFunction {
+    fn from(input: LengthPercentage) -> Self {
+        Self(input.0)
+    }
+}
+impl From<LengthPercentageAuto> for MinTrackSizingFunction {
+    fn from(input: LengthPercentageAuto) -> Self {
+        Self(input.0)
+    }
+}
+impl From<Dimension> for MinTrackSizingFunction {
+    fn from(input: Dimension) -> Self {
+        Self(input.0)
+    }
+}
+#[cfg(feature = "serde")]
+impl<'de> serde::Deserialize<'de> for MinTrackSizingFunction {
+    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
+    where
+        D: serde::Deserializer<'de>,
+    {
+        let inner = CompactLength::deserialize(deserializer)?;
+        // Note: validation intentionally excludes the CALC_TAG as deserializing calc() values is not supported
+        if matches!(
+            inner.tag(),
+            CompactLength::LENGTH_TAG
+                | CompactLength::PERCENT_TAG
+                | CompactLength::AUTO_TAG
+                | CompactLength::MIN_CONTENT_TAG
+                | CompactLength::MAX_CONTENT_TAG
+                | CompactLength::FIT_CONTENT_PX_TAG
+                | CompactLength::FIT_CONTENT_PERCENT_TAG
+        ) {
+            Ok(Self(inner))
+        } else {
+            Err(serde::de::Error::custom("Invalid tag"))
+        }
     }
 }
 
 impl MinTrackSizingFunction {
+    /// An absolute length in some abstract units. Users of Taffy may define what they correspond
+    /// to in their application (pixels, logical pixels, mm, etc) as they see fit.
+    #[inline(always)]
+    pub const fn length(val: f32) -> Self {
+        Self(CompactLength::length(val))
+    }
+
+    /// A percentage length relative to the size of the containing block.
+    ///
+    /// **NOTE: percentages are represented as a f32 value in the range [0.0, 1.0] NOT the range [0.0, 100.0]**
+    #[inline(always)]
+    pub const fn percent(val: f32) -> Self {
+        Self(CompactLength::percent(val))
+    }
+
+    /// The dimension should be automatically computed according to algorithm-specific rules
+    /// regarding the default size of boxes.
+    #[inline(always)]
+    pub const fn auto() -> Self {
+        Self(CompactLength::auto())
+    }
+
+    /// The size should be the "min-content" size.
+    /// This is the smallest size that can fit the item's contents with ALL soft line-wrapping opportunities taken
+    #[inline(always)]
+    pub const fn min_content() -> Self {
+        Self(CompactLength::min_content())
+    }
+
+    /// The size should be the "max-content" size.
+    /// This is the smallest size that can fit the item's contents with NO soft line-wrapping opportunities taken
+    #[inline(always)]
+    pub const fn max_content() -> Self {
+        Self(CompactLength::max_content())
+    }
+
+    /// A `calc()` value. The value passed here is treated as an opaque handle to
+    /// the actual calc representation and may be a pointer, index, etc.
+    ///
+    /// The low 3 bits are used as a tag value and will be returned as 0.
+    #[inline]
+    #[cfg(feature = "calc")]
+    pub fn calc(ptr: *const ()) -> Self {
+        Self(CompactLength::calc(ptr))
+    }
+
+    /// Create a LengthPercentageAuto from a raw `CompactLength`.
+    /// # Safety
+    /// CompactLength must represent a valid variant for LengthPercentageAuto
+    #[allow(unsafe_code)]
+    pub unsafe fn from_raw(val: CompactLength) -> Self {
+        Self(val)
+    }
+
+    /// Get the underlying `CompactLength` representation of the value
+    pub fn into_raw(self) -> CompactLength {
+        self.0
+    }
+
     /// Returns true if the min track sizing function is `MinContent`, `MaxContent` or `Auto`, else false.
     #[inline(always)]
     pub fn is_intrinsic(&self) -> bool {
-        matches!(self, Self::MinContent | Self::MaxContent | Self::Auto)
+        self.0.is_intrinsic()
+    }
+
+    /// Returns true if the min track sizing function is `MinContent` or `MaxContent`, else false.
+    #[inline(always)]
+    pub fn is_min_or_max_content(&self) -> bool {
+        self.0.is_min_or_max_content()
+    }
+
+    /// Returns true if the value is an fr value
+    #[inline(always)]
+    pub fn is_fr(&self) -> bool {
+        self.0.is_fr()
+    }
+
+    /// Returns true if the is `Auto`, else false.
+    #[inline(always)]
+    pub fn is_auto(&self) -> bool {
+        self.0.is_auto()
+    }
+
+    /// Returns true if value is MinContent
+    #[inline(always)]
+    pub fn is_min_content(&self) -> bool {
+        self.0.is_min_content()
+    }
+
+    /// Returns true if value is MaxContent
+    #[inline(always)]
+    pub fn is_max_content(&self) -> bool {
+        self.0.is_max_content()
     }
 
     /// Returns fixed point values directly. Attempts to resolve percentage values against
     /// the passed available_space and returns if this results in a concrete value (which it
     /// will if the available_space is `Some`). Otherwise returns `None`.
     #[inline(always)]
-    pub fn definite_value(self, parent_size: Option<f32>) -> Option<f32> {
-        use MinTrackSizingFunction::*;
-        match self {
-            Fixed(LengthPercentage::Length(size)) => Some(size),
-            Fixed(LengthPercentage::Percent(fraction)) => parent_size.map(|size| fraction * size),
-            MinContent | MaxContent | Auto => None,
+    pub fn definite_value(
+        self,
+        parent_size: Option<f32>,
+        calc_resolver: impl Fn(*const (), f32) -> f32,
+    ) -> Option<f32> {
+        match self.0.tag() {
+            CompactLength::LENGTH_TAG => Some(self.0.value()),
+            CompactLength::PERCENT_TAG => parent_size.map(|size| self.0.value() * size),
+            #[cfg(feature = "calc")]
+            _ if self.0.is_calc() => parent_size.map(|size| calc_resolver(self.0.calc_value(), size)),
+            _ => None,
         }
     }
 
     /// Resolve percentage values against the passed parent_size, returning Some(value)
     /// Non-percentage values always return None.
     #[inline(always)]
-    pub fn resolved_percentage_size(self, parent_size: f32) -> Option<f32> {
-        use MinTrackSizingFunction::*;
-        match self {
-            Fixed(LengthPercentage::Percent(fraction)) => Some(fraction * parent_size),
-            Fixed(LengthPercentage::Length(_)) | MinContent | MaxContent | Auto => None,
-        }
+    pub fn resolved_percentage_size(
+        self,
+        parent_size: f32,
+        calc_resolver: impl Fn(*const (), f32) -> f32,
+    ) -> Option<f32> {
+        self.0.resolved_percentage_size(parent_size, calc_resolver)
     }
 
     /// Whether the track sizing functions depends on the size of the parent node
     #[inline(always)]
     pub fn uses_percentage(self) -> bool {
-        use MinTrackSizingFunction::*;
-        matches!(self, Fixed(LengthPercentage::Percent(_)))
+        #[cfg(feature = "calc")]
+        {
+            matches!(self.0.tag(), CompactLength::PERCENT_TAG) || self.0.is_calc()
+        }
+        #[cfg(not(feature = "calc"))]
+        {
+            matches!(self.0.tag(), CompactLength::PERCENT_TAG)
+        }
     }
 }
 
-/// The sizing function for a grid track (row/column) (either auto-track or template track)
+/// The sizing function for a grid track (row/column)
+///
 /// May either be a MinMax variant which specifies separate values for the min-/max- track sizing functions
 /// or a scalar value which applies to both track sizing functions.
 pub type NonRepeatedTrackSizingFunction = MinMax<MinTrackSizingFunction, MaxTrackSizingFunction>;
@@ -451,7 +884,7 @@ impl NonRepeatedTrackSizingFunction {
     }
     /// Determine whether at least one of the components ("min" and "max") are fixed sizing function
     pub fn has_fixed_component(&self) -> bool {
-        matches!(self.min, MinTrackSizingFunction::Fixed(_)) || matches!(self.max, MaxTrackSizingFunction::Fixed(_))
+        self.min.0.is_length_or_percentage() || self.max.0.is_length_or_percentage()
     }
 }
 impl TaffyAuto for NonRepeatedTrackSizingFunction {
@@ -467,7 +900,7 @@ impl TaffyMaxContent for NonRepeatedTrackSizingFunction {
 }
 impl TaffyFitContent for NonRepeatedTrackSizingFunction {
     fn fit_content(argument: LengthPercentage) -> Self {
-        Self { min: MinTrackSizingFunction::AUTO, max: MaxTrackSizingFunction::FitContent(argument) }
+        Self { min: MinTrackSizingFunction::AUTO, max: MaxTrackSizingFunction::fit_content(argument) }
     }
 }
 impl TaffyZero for NonRepeatedTrackSizingFunction {
@@ -483,9 +916,24 @@ impl FromPercent for NonRepeatedTrackSizingFunction {
         Self { min: MinTrackSizingFunction::from_percent(percent), max: MaxTrackSizingFunction::from_percent(percent) }
     }
 }
-impl FromFlex for NonRepeatedTrackSizingFunction {
-    fn from_flex<Input: Into<f32> + Copy>(flex: Input) -> Self {
-        Self { min: MinTrackSizingFunction::AUTO, max: MaxTrackSizingFunction::from_flex(flex) }
+impl FromFr for NonRepeatedTrackSizingFunction {
+    fn from_fr<Input: Into<f32> + Copy>(flex: Input) -> Self {
+        Self { min: MinTrackSizingFunction::AUTO, max: MaxTrackSizingFunction::from_fr(flex) }
+    }
+}
+impl From<LengthPercentage> for NonRepeatedTrackSizingFunction {
+    fn from(input: LengthPercentage) -> Self {
+        Self { min: input.into(), max: input.into() }
+    }
+}
+impl From<LengthPercentageAuto> for NonRepeatedTrackSizingFunction {
+    fn from(input: LengthPercentageAuto) -> Self {
+        Self { min: input.into(), max: input.into() }
+    }
+}
+impl From<Dimension> for NonRepeatedTrackSizingFunction {
+    fn from(input: Dimension) -> Self {
+        Self { min: input.into(), max: input.into() }
     }
 }
 
@@ -523,7 +971,7 @@ impl core::fmt::Display for InvalidStringRepetitionValue {
         f.write_str("&str can only be converted to GridTrackRepetition if it's value is 'auto-fit' or 'auto-fill'")
     }
 }
-impl<'a> TryFrom<&'a str> for GridTrackRepetition {
+impl TryFrom<&str> for GridTrackRepetition {
     type Error = InvalidStringRepetitionValue;
     fn try_from(value: &str) -> Result<Self, InvalidStringRepetitionValue> {
         match value {
@@ -578,9 +1026,9 @@ impl FromPercent for TrackSizingFunction {
         Self::Single(NonRepeatedTrackSizingFunction::from_percent(percent))
     }
 }
-impl FromFlex for TrackSizingFunction {
-    fn from_flex<Input: Into<f32> + Copy>(flex: Input) -> Self {
-        Self::Single(NonRepeatedTrackSizingFunction::from_flex(flex))
+impl FromFr for TrackSizingFunction {
+    fn from_fr<Input: Into<f32> + Copy>(flex: Input) -> Self {
+        Self::Single(NonRepeatedTrackSizingFunction::from_fr(flex))
     }
 }
 impl From<MinMax<MinTrackSizingFunction, MaxTrackSizingFunction>> for TrackSizingFunction {
@@ -588,30 +1036,3 @@ impl From<MinMax<MinTrackSizingFunction, MaxTrackSizingFunction>> for TrackSizin
         Self::Single(input)
     }
 }
-
-// Grid extensions to the Style struct
-impl Style {
-    /// Get a grid item's row or column placement depending on the axis passed
-    pub(crate) fn grid_template_tracks(&self, axis: AbsoluteAxis) -> &GridTrackVec<TrackSizingFunction> {
-        match axis {
-            AbsoluteAxis::Horizontal => &self.grid_template_columns,
-            AbsoluteAxis::Vertical => &self.grid_template_rows,
-        }
-    }
-
-    /// Get a grid item's row or column placement depending on the axis passed
-    pub(crate) fn grid_placement(&self, axis: AbsoluteAxis) -> Line<GridPlacement> {
-        match axis {
-            AbsoluteAxis::Horizontal => self.grid_column,
-            AbsoluteAxis::Vertical => self.grid_row,
-        }
-    }
-
-    /// Get a grid container's align-content or justify-content alignment depending on the axis passed
-    pub(crate) fn grid_align_content(&self, axis: AbstractAxis) -> AlignContent {
-        match axis {
-            AbstractAxis::Inline => self.justify_content.unwrap_or(AlignContent::Stretch),
-            AbstractAxis::Block => self.align_content.unwrap_or(AlignContent::Stretch),
-        }
-    }
-}
