@@ -26,30 +26,6 @@ where
     TrackSizingFunction::Repeat(repetition_kind.try_into().unwrap(), track_list)
 }
 
-#[cfg(feature = "grid")]
-#[cfg(test)]
-mod repeat_fn_tests {
-    use super::repeat;
-    use crate::style::{GridTrackRepetition, NonRepeatedTrackSizingFunction, TrackSizingFunction};
-
-    const TEST_VEC: Vec<NonRepeatedTrackSizingFunction> = Vec::new();
-
-    #[test]
-    fn test_repeat_u16() {
-        assert_eq!(repeat(123, TEST_VEC), TrackSizingFunction::Repeat(GridTrackRepetition::Count(123), TEST_VEC));
-    }
-
-    #[test]
-    fn test_repeat_auto_fit_str() {
-        assert_eq!(repeat("auto-fit", TEST_VEC), TrackSizingFunction::Repeat(GridTrackRepetition::AutoFit, TEST_VEC));
-    }
-
-    #[test]
-    fn test_repeat_auto_fill_str() {
-        assert_eq!(repeat("auto-fill", TEST_VEC), TrackSizingFunction::Repeat(GridTrackRepetition::AutoFill, TEST_VEC));
-    }
-}
-
 #[cfg(feature = "grid")]
 /// Returns a grid template containing `count` evenly sized tracks
 pub fn evenly_sized_tracks(count: u16) -> Vec<TrackSizingFunction> {
@@ -80,7 +56,7 @@ pub fn span<T: TaffyGridSpan>(span: u16) -> T {
 }
 /// Trait to abstract over grid span values
 pub trait TaffyGridSpan {
-    /// Converts an iu6 into Self
+    /// Converts an u16 into Self
     fn from_span(span: u16) -> Self;
 }
 
@@ -535,12 +511,36 @@ impl<T: FromPercent> Rect<T> {
 
 /// Create a `Fraction` track sizing function (`fr` in CSS)
 #[cfg(feature = "grid")]
-pub fn fr<Input: Into<f32> + Copy, T: FromFlex>(flex: Input) -> T {
-    T::from_flex(flex)
+pub fn fr<Input: Into<f32> + Copy, T: FromFr>(flex: Input) -> T {
+    T::from_fr(flex)
 }
 
 /// Trait to create constant percent values from plain numbers
-pub trait FromFlex {
+pub trait FromFr {
     /// Converts into an `Into<f32>` into Self
-    fn from_flex<Input: Into<f32> + Copy>(flex: Input) -> Self;
+    fn from_fr<Input: Into<f32> + Copy>(flex: Input) -> Self;
+}
+
+#[cfg(feature = "grid")]
+#[cfg(test)]
+mod repeat_fn_tests {
+    use super::repeat;
+    use crate::style::{GridTrackRepetition, NonRepeatedTrackSizingFunction, TrackSizingFunction};
+
+    const TEST_VEC: Vec<NonRepeatedTrackSizingFunction> = Vec::new();
+
+    #[test]
+    fn test_repeat_u16() {
+        assert_eq!(repeat(123, TEST_VEC), TrackSizingFunction::Repeat(GridTrackRepetition::Count(123), TEST_VEC));
+    }
+
+    #[test]
+    fn test_repeat_auto_fit_str() {
+        assert_eq!(repeat("auto-fit", TEST_VEC), TrackSizingFunction::Repeat(GridTrackRepetition::AutoFit, TEST_VEC));
+    }
+
+    #[test]
+    fn test_repeat_auto_fill_str() {
+        assert_eq!(repeat("auto-fill", TEST_VEC), TrackSizingFunction::Repeat(GridTrackRepetition::AutoFill, TEST_VEC));
+    }
 }
